@@ -30,11 +30,7 @@
 #include "openssl-include.h"
 #include <glib/gi18n-lib.h>
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined (LIBRESSL_VERSION_NUMBER)
-#  define DEFAULT_CIPHER_LIST "HIGH:!DSS:!aNULL@STRENGTH"
-#else
-#  define DEFAULT_CIPHER_LIST "PROFILE=SYSTEM"
-#endif
+#define DEFAULT_CIPHER_LIST "HIGH:!DSS:!aNULL@STRENGTH"
 
 typedef struct _GTlsServerConnectionOpensslPrivate
 {
@@ -78,6 +74,67 @@ g_tls_server_connection_openssl_finalize (GObject *object)
   G_OBJECT_CLASS (g_tls_server_connection_openssl_parent_class)->finalize (object);
 }
 
+static gboolean
+ssl_set_certificate (SSL              *ssl,
+                     GTlsCertificate  *cert,
+                     GError          **error)
+{
+  EVP_PKEY *key;
+  X509 *x;
+  GTlsCertificate *issuer;
+
+  key = g_tls_certificate_openssl_get_key (G_TLS_CERTIFICATE_OPENSSL (cert));
+
+  if (key == NULL)
+    {
+      g_set_error_literal (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
+                           _("Certificate has no private key"));
+      return FALSE;
+    }
+
+  /* Note, order is important. If a certificate has been set previously,
+   * OpenSSL requires that the new certificate is set _before_ the new
+   * private key is set. */
+  x = g_tls_certificate_openssl_get_cert (G_TLS_CERTIFICATE_OPENSSL (cert));
+  if (SSL_use_certificate (ssl, x) <= 0)
+    {
+      g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
+                   _("There is a problem with the certificate: %s"),
+                   ERR_error_string (ERR_get_error (), NULL));
+      return FALSE;
+    }
+
+  if (SSL_use_PrivateKey (ssl, key) <= 0)
+    {
+      g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
+                   _("There is a problem with the certificate private key: %s"),
+                   ERR_error_string (ERR_get_error (), NULL));
+      return FALSE;
+    }
+
+  if (SSL_clear_chain_certs (ssl) == 0)
+    g_warning ("There was a problem clearing the chain certificates: %s",
+               ERR_error_string (ERR_get_error (), NULL));
+
+  /* Add all the issuers to create the full certificate chain */
+  for (issuer = g_tls_certificate_get_issuer (G_TLS_CERTIFICATE (cert));
+       issuer != NULL;
+       issuer = g_tls_certificate_get_issuer (issuer))
+    {
+      X509 *issuer_x;
+
+      /* Be careful here and duplicate the certificate since the context
+       * will take the ownership
+       */
+      issuer_x = X509_dup (g_tls_certificate_openssl_get_cert (G_TLS_CERTIFICATE_OPENSSL (issuer)));
+      if (SSL_add0_chain_cert (ssl, issuer_x) == 0)
+        g_warning ("There was a problem adding the chain certificate: %s",
+                   ERR_error_string (ERR_get_error (), NULL));
+    }
+
+  return TRUE;
+}
+
 static void
 g_tls_server_connection_openssl_get_property (GObject    *object,
                                               guint       prop_id,
@@ -94,7 +151,7 @@ g_tls_server_connection_openssl_get_property (GObject    *object,
     case PROP_AUTHENTICATION_MODE:
       g_value_set_enum (value, priv->authentication_mode);
       break;
-      
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -172,15 +229,19 @@ g_tls_server_connection_openssl_get_ssl (GTlsConnectionOpenssl *connection)
   return priv->ssl;
 }
 
-static SSL_CTX *
-g_tls_server_connection_openssl_get_ssl_ctx (GTlsConnectionOpenssl *connection)
+static void
+on_certificate_changed (GObject    *object,
+                        GParamSpec *spec,
+                        gpointer    user_data)
 {
-  GTlsServerConnectionOpenssl *server = G_TLS_SERVER_CONNECTION_OPENSSL (connection);
-  GTlsServerConnectionOpensslPrivate *priv;
+  SSL *ssl;
+  GTlsCertificate *cert;
 
-  priv = g_tls_server_connection_openssl_get_instance_private (server);
+  ssl = g_tls_server_connection_openssl_get_ssl (G_TLS_CONNECTION_OPENSSL (object));
+  cert = g_tls_connection_get_certificate (G_TLS_CONNECTION (object));
 
-  return priv->ssl_ctx;
+  if (ssl && cert)
+    ssl_set_certificate (ssl, cert, NULL);
 }
 
 static void
@@ -197,7 +258,6 @@ g_tls_server_connection_openssl_class_init (GTlsServerConnectionOpensslClass *kl
   base_class->handshake = g_tls_server_connection_openssl_handshake;
 
   connection_class->get_ssl = g_tls_server_connection_openssl_get_ssl;
-  connection_class->get_ssl_ctx = g_tls_server_connection_openssl_get_ssl_ctx;
 
   g_object_class_override_property (gobject_class, PROP_AUTHENTICATION_MODE, "authentication-mode");
 }
@@ -226,8 +286,9 @@ ssl_info_callback (const SSL *ssl,
 }
 #endif
 
-static void
-set_cipher_list (GTlsServerConnectionOpenssl *server)
+static gboolean
+set_cipher_list (GTlsServerConnectionOpenssl  *server,
+                 GError                      **error)
 {
   GTlsServerConnectionOpensslPrivate *priv;
   const gchar *cipher_list;
@@ -238,7 +299,15 @@ set_cipher_list (GTlsServerConnectionOpenssl *server)
   if (cipher_list == NULL)
     cipher_list = DEFAULT_CIPHER_LIST;
 
-  SSL_CTX_set_cipher_list (priv->ssl_ctx, cipher_list);
+  if (!SSL_CTX_set_cipher_list (priv->ssl_ctx, cipher_list))
+    {
+      g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_MISC,
+                   _("Could not create TLS context: %s"),
+                   ERR_error_string (ERR_get_error (), NULL));
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 #ifdef SSL_CTX_set1_sigalgs_list
@@ -298,6 +367,9 @@ g_tls_server_connection_openssl_initable_init (GInitable       *initable,
       return FALSE;
     }
 
+  if (!set_cipher_list (server, error))
+    return FALSE;
+
   /* Only TLS 1.2 or higher */
   options = SSL_OP_NO_TICKET |
             SSL_OP_NO_COMPRESSION |
@@ -317,60 +389,8 @@ g_tls_server_connection_openssl_initable_init (GInitable       *initable,
 
   SSL_CTX_set_options (priv->ssl_ctx, options);
 
-  cert = g_tls_connection_get_certificate (G_TLS_CONNECTION (initable));
-  if (cert != NULL)
-    {
-      EVP_PKEY *key;
-      X509 *x;
-      GTlsCertificate *issuer;
-
-      key = g_tls_certificate_openssl_get_key (G_TLS_CERTIFICATE_OPENSSL (cert));
-
-      if (key == NULL)
-        {
-          g_set_error_literal (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
-                               _("Certificate has no private key"));
-          return FALSE;
-        }
-
-      if (SSL_CTX_use_PrivateKey (priv->ssl_ctx, key) <= 0)
-        {
-          g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
-                       _("There is a problem with the certificate private key: %s"),
-                       ERR_error_string (ERR_get_error (), NULL));
-          return FALSE;
-        }
-
-      x = g_tls_certificate_openssl_get_cert (G_TLS_CERTIFICATE_OPENSSL (cert));
-      if (SSL_CTX_use_certificate (priv->ssl_ctx, x) <= 0)
-        {
-          g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE,
-                       _("There is a problem with the certificate: %s"),
-                       ERR_error_string (ERR_get_error (), NULL));
-          return FALSE;
-        }
-
-      /* Add all the issuers to create the full certificate chain */
-      for (issuer = g_tls_certificate_get_issuer (G_TLS_CERTIFICATE (cert));
-           issuer != NULL;
-           issuer = g_tls_certificate_get_issuer (issuer))
-        {
-          X509 *issuer_x;
-
-          /* Be careful here and duplicate the certificate since the context
-           * will take the ownership
-           */
-          issuer_x = X509_dup (g_tls_certificate_openssl_get_cert (G_TLS_CERTIFICATE_OPENSSL (issuer)));
-          if (!SSL_CTX_add_extra_chain_cert (priv->ssl_ctx, issuer_x))
-            g_warning ("There was a problem adding the extra chain certificate: %s",
-                       ERR_error_string (ERR_get_error (), NULL));
-        }
-    }
-
   SSL_CTX_add_session (priv->ssl_ctx, priv->session);
 
-  set_cipher_list (server);
-
 #ifdef SSL_CTX_set1_sigalgs_list
   set_signature_algorithm_list (server);
 #endif
@@ -407,12 +427,18 @@ g_tls_server_connection_openssl_initable_init (GInitable       *initable,
       return FALSE;
     }
 
+  cert = g_tls_connection_get_certificate (G_TLS_CONNECTION (initable));
+  if (cert != NULL && !ssl_set_certificate (priv->ssl, cert, error))
+    return FALSE;
+
   SSL_set_accept_state (priv->ssl);
 
   if (!g_tls_server_connection_openssl_parent_initable_iface->
       init (initable, cancellable, error))
     return FALSE;
 
+  g_signal_connect (server, "notify::certificate", G_CALLBACK (on_certificate_changed), NULL);
+
   return TRUE;
 }
 
